import * as FieldScript from './FieldScript.js';
import { SafeInputs } from './crud-mongo-SafeInputs.js';

export function str(extraOptions={}) {
  return stringField(extraOptions)
}

export function int(extraOptions={}) {
  return intField(extraOptions)
}

export function bool(extraOptions={}) {
  return booleanField(extraOptions)
}

export function list(extraOptions={}) {
  return listField(extraOptions)
}

export function stringField(extraOptions={}) {
    return { fieldType:'string', extraOptions } 
}
export function booleanField(extraOptions={}) {
    return { fieldType:'boolean', extraOptions} 
}
export function intField(extraOptions={}) {
    return { fieldType:'int', extraOptions} 
}
export function objectField(extraOptions={}) {
    return { fieldType:'obj', extraOptions} 
}
export function listField(extraOptions={}) {
    return { fieldType:'list', extraOptions} 
}

export function createNew(model,obj) {
  return check(model,obj);
}

export function toJson(models) {
  return JSON.stringify(toObj(models));
}

toObj({notes:{title:str}})
export function toObj(models) {
  const freshObj = {};
  const keys = Object.keys(models);
  for(const key of keys) {
      let theModel = models[key];
      let freshProp = {};
      const props = Object.keys(theModel);
    
      for(const propKey of props) {
        let theProp = theModel[propKey];
          if(typeof theProp === "function") {
            theProp = theProp();
          }
        freshProp[propKey] = theProp;
      }
    
     freshObj[key] = freshProp
  }

  return freshObj;
}

/*
// test 
const test = check({title:stringField(),order:stringField()},{title:'1',order:1})
console.log({test})
//*/
export function check(model, obj) {
  const freshObj = {};
  const keys = Object.keys(model);
  ///console.log({keys})
  for(const key of keys) {
      //console.log({key})
      let theModel = model[key];
      if(typeof theModel === "function") {
        theModel = theModel();
      }
    
      if(obj[key] === undefined && "default" in theModel.extraOptions) {
          obj[key] = theModel.extraOptions.default; // support default values in check
      }
      else if(obj[key] === undefined) throw new Error('key ' + key + ' is undefined');

      if(theModel.fieldType=== stringField().fieldType) {
          freshObj[key] = FieldScript.stringField(obj[key],{name:key})
      } 
      else if(theModel.fieldType=== booleanField().fieldType) {
          freshObj[key] = FieldScript.booleanField(obj[key],{name:key})
      } 
      else if(theModel.fieldType=== intField().fieldType) {
          freshObj[key] = FieldScript.intField(obj[key],{name:key})
      } 
      else if(theModel.fieldType=== objectField().fieldType) {
          freshObj[key] = FieldScript.objectField(obj[key],{name:key})
      } 
      else if(theModel.fieldType=== listField().fieldType) {
          freshObj[key] = FieldScript.listField(obj[key],{name:key})
      } 
      else {
          throw new Error('unknown ModelScript type');
      }

  }
    return freshObj;
}

export function from(model, obj, skipKeys=false, mode='mongodb') {
  for(const key of Object.keys(model)) {
      if(!(key in obj)) {
          throw new Error("key " + key + " not found in obj");
      }
  }
  
    return new SafeInputs(obj, model,skipKeys, mode);
}



/*
// test
const models = {
    notes: {
        name: stringField(true),
     },
     contacts: {
         title: stringField(true),
     }
}
console.log('models',models)
console.log('models.contacts',models.contacts)
console.log(from(models.notes, {"name":"test"}));

console.log(from(models.notes, {"$name":"$test"})); // exp missing key error "name" to be thrown
//*/